# -*- coding:utf-8 -*-
import re
import sys
try:
    from pygments import highlight
    from pygments import util
    from pygments.lexers import get_lexer_for_filename
    from pygments.formatters import HtmlFormatter
    has_pygments = True
except ImportError:
    has_pygments = False

# show error message in web page.
def raise_pygments_error(error_message):
    return "<span class=\"k\">RedmineSystemError: " + str(error_message) + "</span>"

# get rails repository file content string from ruby IO.popen pipe.
def get_file_name_and_file_content(fileobject=sys.stdin):
    scm_file_content_str = ""
    # Readlines of file content from fileobject
    for item in fileobject:
        scm_file_content_str += item
    try:
        # the file_name is stored in first line of scm_file_content_str
        file_name = re.findall('\A.*\\n', scm_file_content_str)[0]
        file_name = re.sub('\n','', file_name)
        # because *.S is both in pygments.lexers.asm and pygments.lexers.math,
        # so, make *.s or *.S as *.asm file not MuPAD file(math).
        if re.match('.*\.s$|.*\.S$', file_name):
            file_name = "default.asm"
        # delete first line and whitespaces at start.
        scm_file_content_str = re.sub('\A.*\\n', '', scm_file_content_str)
    except Exception, e:
        file_name = "default.txt"
        scm_file_content_str = e
    return file_name, scm_file_content_str

def get_html_output(file_name, file_content):
    try:
        # if the repository encoding isn't empty.
        file_content.decode("utf-8")
        lexer = get_lexer_for_filename(file_name, encoding="utf-8")
    except:
        lexer = get_lexer_for_filename(file_name)
    formatter = HtmlFormatter()
    # delete the whitespaces at the end of highlight(file_content, lexer, formatter)
    output = re.sub('\n.*\n$', '', highlight(file_content, lexer, formatter))
    try:
        # delete <div> tag in the first line of output.
        pattern_str = '^<div\sclass=\"highlight\"><pre>(.*</span>)'
        replace_fregment = re.match(pattern_str, output).group(1)
        output = re.sub(pattern_str, replace_fregment, output)
    except:
        pass
    return output

if __name__ == '__main__':
    file_name, scm_file_content_str = get_file_name_and_file_content(sys.stdin)
    if not has_pygments:
        result = raise_pygments_error("Pygments not found")
    else:
        try:
            result = get_html_output(file_name, scm_file_content_str)
        except util.ClassNotFound:
            result = get_html_output("default.txt", scm_file_content_str)
        except Exception, e:
            result = raise_pygments_error(e)
    # write result into stdout, ruby IO.popen will get the stdout.
    # Notice: Make sure you just write the file content html formatter strings
    #       to stdout, don't print any other string.
    print result.encode("utf-8")